/**
*
* $Id: Nexus.cpp,v 0.1 2004/11/20 18:03:45 erv Exp $
* (c) 2004 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/

#include "Nexus.h"



// 1. vertical lift component (wings and body)
// Borrowed from Mike Norman's Shuttle code

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	//                              -180 -165  -150 -135  -120 -105 -90 -75.0  -60.0  -45.0  -30.0  -15.0   0.0   15.0      30.0     45.0  60.0  75.0  90.0 105.0  120.0  135.0  150.0  165.0 180.0  
	static const double CL[nabsc] = {0.1, 0.12, 0.14, 0.14, 0.12, 0.1, 0, -0.08, -0.11, -0.13,  -0.15,  -0.18,  0.0, 0.2955,    0.26,   0.18, 0.14, 0.10, 0.0, -0.10, -0.16, -0.19, -0.14, -0.1, 0.1};
	static const double CM[nabsc] = {  0,    0,   0,   0,    0,   0, 0,     0,     0,  0.002,  0.004, 0.0025,0.0012,      0,-0.0012,-0.0007,    0,    0,   0,     0,     0,     0,     0,    0,   0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	// change for drag increase at high Mach:
	// Subsonic Equivalents

//	double Cl0, Cl1, Cd0, Cd1, Cd2;
/*	if(M>=5.0) {
		Cl0 = -0.187040;
		Cl1 = 1.675600;
		Cd0 = 0.068540;
		Cd1 = -0.21767;
		Cd2 = 1.3632;
		*cl = Cl0 + Cl1*aoa;
		*cd = Cd0 + Cd1*aoa + Cd2*aoa* aoa;
	}
*/ 	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cm = (CM[idx] + (CM[idx+1]-CM[idx])*d);
	if(M>=5.0) {
		*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
 		*cd = 0.30 + 0.75*oapiGetInducedDrag (*cl, 1.0, 0.65);
	}
	if(M<1.25) { 
 		*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
 		*cd = 0.04 + 0.33*oapiGetInducedDrag (*cl, 1.0, 0.82);
	}
	else if(M<5.0) { 
 		*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
 		*cd = 0.05 + 0.8*oapiGetInducedDrag (*cl, 1.0, 0.65);
// 		*cd = 0.06 + oapiGetInducedDrag (*cl, 2.266, 0.6);
	}

}


// 2. horizontal lift component (vertical stabilisers and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	int i;
	const int nabsc = 8;
	static const double BETA[nabsc] = {-180*RAD,-135*RAD,-90*RAD,-45*RAD,45*RAD,90*RAD,135*RAD,180*RAD};
	static const double CL[nabsc]   = {       0,    +0.15,      0,   -0.15,  +0.15,     0,   -0.15,      0};
	for (i = 0; i < nabsc-1 && BETA[i+1] < beta; i++);
	*cl = CL[i] + (CL[i+1]-CL[i]) * (beta-BETA[i]) / (BETA[i+1]-BETA[i]);
	*cm = 0.0;
	*cd = 0.015 + oapiGetInducedDrag (*cl, 1.5, 0.6) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
	
	if (M > 1.0) *cd *= 2.2;
	else *cd *= 0.6;
}


NEXUS::NEXUS (OBJHANDLE hObj, int fmodel)
: COMMON_VESSEL_EX (hObj)
{
	hNexus = oapiLoadMeshGlobal ("Nexus_Stage1");
	hNexusR = oapiLoadMeshGlobal ("Nexus_Retrosection");
	hChute = oapiLoadMeshGlobal ("NexusChute");
	tex = oapiRegisterExhaustTexture("Exhaust_atsme");

	/* set the propellant handle to NULL so I can check if I need to 
	/* create it in the SetMainStage() function. */
	ph_first=NULL; 
	ph_att=NULL; 
	ph_braking=NULL;


	char classname[30];
	int i;
	strncpy( classname, GetClassName(), 29 );
	classname[29]=0;
	for( i = 0 ; i < strlen( classname ) ; i++ ) {
		classname[i] = toupper(classname[i]);
	}
	if( strstr( classname, "NEXUSR" ) )
		stage = 0;
	else
		stage = 1;

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GDPS requests. Have it run
	/* in the constructor so it can set up things like the InitFuelLevel
	/* (initial fuel level) variable, and load the payloads */
	CheckCentralMessage();
	
	brakes_status=DOOR_CLOSED;
	brakes_proc=0.0;
	chute_status=DOOR_CLOSED;
	chute_proc=0.0;
	deflate_status=DOOR_CLOSED;
	deflate_proc=0.0;
}


NEXUS::~NEXUS()
{

}
void NEXUS::UpdateAnimations()
{	

		// animate speedbrakes
		if (brakes_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * BRAKES_OPERATING_SPEED;
			if (brakes_status == DOOR_CLOSING) {
				if (brakes_proc > 0.0)
					brakes_proc = max (0.0, brakes_proc-da);
				else {
					brakes_status = DOOR_CLOSED;
				}
			} else  { // door opening
				if (brakes_proc < 1.0)
					brakes_proc = min (1.0, brakes_proc+da);
				else {
					brakes_status = DOOR_OPEN;
				}
			}
			SetAnimation (anim_brakes, brakes_proc);
		}

		// animate parachute
		if (chute_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * CHUTE_OPERATING_SPEED;
			if (chute_status == DOOR_CLOSING) {
				if (chute_proc > 0.0)
					chute_proc = max (0.0, chute_proc-da);
				else {
					chute_status = DOOR_CLOSED;
				}
			} else  { // door opening
				if (chute_proc < 1.0)
					chute_proc = min (1.0, chute_proc+da);
				else {
					chute_status = DOOR_OPEN;
				}
			}
			SetAnimation (anim_chute, chute_proc);
		}

		// animate parachute deflation
		if (deflate_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * DEFLATE_SPEED;
			if (deflate_status == DOOR_CLOSING) {
				if (deflate_proc > 0.0)
					deflate_proc = max (0.0, deflate_proc-da);
				else {
					deflate_status = DOOR_CLOSED;
				}
			} else  { // door opening
				if (deflate_proc < 1.0)
					deflate_proc = min (1.0, deflate_proc+da);
				else {
					deflate_status = DOOR_OPEN;
				}
			}
			SetAnimation (anim_deflate, deflate_proc);
		}

}

void NEXUS::ActivateBrakes (DoorStatus action)
{
	brakes_status = action;
}

void NEXUS::RevertBrakes ()
{
	ActivateBrakes (brakes_status == DOOR_CLOSED || brakes_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void NEXUS::ActivateChute (DoorStatus action)
{
	chute_status = action;
}

void NEXUS::RevertChute ()
{
	ActivateChute (chute_status == DOOR_CLOSED || chute_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void NEXUS::ActivateDeflate (DoorStatus action)
{
	deflate_status = action;
}

void NEXUS::RevertDeflate ()
{
	ActivateDeflate (deflate_status == DOOR_CLOSED || deflate_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void NEXUS::MakeAnimations()
{
//	if (stage == 0) return;

	AnimationsMade = true;

	anim_brakes = CreateAnimation (0);
	anim_chute = CreateAnimation (0);
	anim_deflate = CreateAnimation (0);


/* Known Mesh Groups:

12 flap1
13 flap2
14 flap3
05 flap4 

  */

		// ***** Speedbrake animation *****

	static UINT Rudder1Grp[1] = {12};
	static MGROUP_ROTATE Rudder1 (0, Rudder1Grp, 1,
		_V( 22.0,0.0,12.255), _V( 0,-1,0), (float)(45*RAD));
	static UINT Rudder2Grp[1] = {13};
	static MGROUP_ROTATE Rudder2 (0, Rudder2Grp, 1,
		_V(0.0,22.0,12.255), _V(1,0,0), (float)(45*RAD));
	static UINT Rudder3Grp[1] = {14};
	static MGROUP_ROTATE Rudder3 (0, Rudder3Grp, 1,
		_V(-22.0,0.0,12.255), _V(0,1,0), (float)(45*RAD));
	static UINT Rudder4Grp[1] = {5};
	static MGROUP_ROTATE Rudder4 (0, Rudder4Grp, 1,
		_V(0.0,-22.0,12.255), _V(-1,0,0 ), (float)(45*RAD));


	AddAnimationComponent (anim_brakes, 0, 1, &Rudder1);
	AddAnimationComponent (anim_brakes, 0, 1, &Rudder2);
	AddAnimationComponent (anim_brakes, 0, 1, &Rudder3);
	AddAnimationComponent (anim_brakes, 0, 1, &Rudder4);


	static UINT ExpandGrp[4] = {0,1,2,3};
		static MGROUP_SCALE Expand (1, ExpandGrp, 4,
			_V(0,0,0), _V(600.0,600.0,600.0));
	AddAnimationComponent (anim_chute, 0.05, 0.95, &Expand);


	static UINT TiltGrp[4] = {0,1,2,3};
	static MGROUP_ROTATE Tilt (1, TiltGrp, 4,
		_V( 0.0,0.0, 0.0), _V(0.5,0.866,0), (float)(45*RAD));
	static UINT DeflateGrp[4] = {0,1,2,3};
	static MGROUP_SCALE Deflate (1, DeflateGrp, 4,
		_V(0,0,0), _V(1.0,1.0,0.0001));
	AddAnimationComponent (anim_deflate, 0.02, 0.5, &Tilt);
	AddAnimationComponent (anim_deflate, 0.2, 0.95, &Deflate);

}

void NEXUS::TimeStep(  double simt )
{
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GPDS requests and returns a bool (true/false)
	/* if any GPDS requests have been implemented.
	/* If any have (eg adding a new payload), you will need to re-draw your ship, hence the
	/* call to the (repeatable) SetMainStage(); */
	if( CheckCentralMessage() )
		SetMainStage();

//	sprintf(oapiDebugString(), "L/D = %0.4f",(GetLift()/GetDrag()));
	if (stage > 0)
		if (brakes_status == DOOR_CLOSED) RevertBrakes();
	
	if (stage == 0)
		brakes_proc = 0.0;

	if (GetAltitude() < 7000.0)
		if (chute_status == DOOR_CLOSED) {
			RevertChute();
			stage++;
			SetMainStage();
		}

	if (GetAltitude() < 50.0)
		SetThrusterGroupLevel(THGROUP_RETRO,1.0);

	if (stage > 0)
		UpdateAnimations();

	if (((GetThrusterMax0(th_first[0]))/GetMass()) > 39.24) 
		SetThrusterMax0(th_first[0],0.98*GetThrusterMax0(th_first[0]));


	if (stage == 0)
		if (GetAltitude() < 100000.0) 
			JettisonRetro();

	if (stage == 1)
		if (GetAltitude() < 9000.0)
			SetPropellantMass(ph_first,0.0);

	if (stage == 0)
		if (GetPropellantMass(ph_first) == 0) 
			JettisonRetro();
		
	if (stage == 1)
		if (GetPropellantMass(ph_first) == 0) {
			stage++;
			SetMainStage();
		}

	if ((GetAltitude() < 10.0) && (deflate_proc == 0.0)) 
		RevertDeflate();

}

/* CVE-lite function ! */
/* IMPORTANT */
/* VisualCreated() is a function passed through from CVE-lite as a direct replacement
/* for the DLLCLBK ovcVisualCreated() function.
/* You can't use ovcVisualCreated() with CVE-lite, so use this instead! */

void NEXUS::VisualCreated ( VESSEL *vessel, VISHANDLE vis, int refcount) {
	/* local equivalent to the DLLCLBK ovcVisualCreated() */
	SetMainStage(); // this is a good place to ask for the initial setup.
}


void NEXUS::SetClassCaps(  FILEHANDLE cfg )
{
	/* ensure the propellent handle is set to NULL before creation */
	ph_first=NULL;
	ph_att=NULL;
	ph_braking=NULL;
	
	CreateControlSurface (AIRCTRL_ELEVATORTRIM, 1200,  0.8, _V(  0,10.0,-30.0), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_ELEVATOR,		250,  0.5, _V(  0,0.0,-50.0), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_RUDDER,       250,  0.01, _V(  0,0.0,-50.0), AIRCTRL_AXIS_YPOS);
	SetControlSurfaceLevel(AIRCTRL_ELEVATORTRIM,1.0);
	//                  alignment      position      func      chord  S   AR   
		CreateAirfoil (LIFT_VERTICAL, _V(0,0.0,-1.5), VLiftCoeff, 200.0, 820.9, 1);
		// wing and body lift+drag components

		CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,-60.6), HLiftCoeff, 120.0, 300.5, 1);
		// vertical stabiliser and body lift and drag components

	SetMainStage();
}

void NEXUS::SaveState( FILEHANDLE scn )
{
	char cbuf[256];

	SaveDefaultState (scn);
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Call this after the normal SaveDefautState() to record attached payloads
	/* it the scenario file */

	oapiWriteScenario_int (scn, "CONFIGURATION", stage);
	sprintf (cbuf, "%d %0.4f", brakes_status, brakes_proc);
		oapiWriteScenario_string (scn, "BRAKES", cbuf);
	sprintf (cbuf, "%d %0.4f", chute_status, chute_proc);
		oapiWriteScenario_string (scn, "CHUTE", cbuf);

	/* save payload lines */
	SaveDefaultStateEx(scn);
}

void NEXUS::LoadState(  FILEHANDLE scn, void *vs )
{
    char *line;

	while (oapiReadScenario_nextline (scn, line)) 
	{
		/* CVE-lite function ! */
		/* IMPORTANT */
		/* Call this for every line in the scenario file. It will automatically
		/* deduce which lines are for it, and load the payloads from the file. */

		/* load payload lines */
		if (!strnicmp (line, "CONFIGURATION", 13)) {
		    sscanf (line+13, "%d", &stage);
		} else if (!strnicmp (line, "BRAKES", 6)) {
			sscanf (line+6, "%d%lf", &brakes_status, &brakes_proc);
		} else if (!strnicmp (line, "CHUTE", 5)) {
			sscanf (line+5, "%d%lf", &chute_status, &chute_proc);
		}		else ParsePayloadString(line);


		ParseScenarioLineEx (line, vs);
	}

	/* always set the stage whenever something may have changed */
	SetMainStage();
	RevertBrakes();
	RevertBrakes();
	RevertChute();
	RevertChute();
	RevertDeflate();
	RevertDeflate();
}

/* IMPORTANT */
/* Always have a function that you can call as many times as you like that will re-draw
/* all you meshes, reset all your thrusters and check to make sure the propellent is
/* there. This should be called after any configuration change, or any GDPS line (see
/* TimeStep(), above) */

void NEXUS::SetMainStage( )
{

	const VECTOR3 VECTOR_ZERO = _V(0,0,0);

	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

//	ClearAirfoilDefinitions();

//	if ((stage == 1) || (stage == 2)) {
	//                  alignment      position      func      chord  S   AR   
		CreateAirfoil (LIFT_VERTICAL, _V(0,0.0,-1.1), VLiftCoeff, 250.0, 1020.9, 4);
		// wing and body lift+drag components

		CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,-60.6), HLiftCoeff, 120.0, 200.5, 4);
		// vertical stabiliser and body lift and drag components
//	}
	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
	ClearThrusterDefinitions();

	PARTICLESTREAMSPEC contrail = 
	{
		0, 25.0, 7, 350, 0.3, 4.0, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");
	SetSize (100.0);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */

	if (stage == 0)
		SetEmptyMass ( NEXUS_DRYMASS + 20000.0 + CalculateMass() );
	else
		SetEmptyMass ( NEXUS_DRYMASS + CalculateMass() );


	SetTouchdownPoints (_V(0,4.4,2.1), _V(-4.4,0,2.1), _V(4.4,0,2.1));
	SetPMI (_V(409,415,476));
	SetCrossSections (_V(1469.0,1484.0,2013.0));
	if (chute_proc > 0.0)
		SetCrossSections (_V(1469.0,1484.0,20013.0));
	SetCW (1.1,1.1,0.9,0.9);
	SetRotDrag (_V(1.1,1.1,0.7));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
//	SetLiftCoeffFunc (0); 
//	if (stage == 1)
		SetTrimScale (0.025);

	ClearVariableDragElements();
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	Draw();


	/* we're not using COMMON_MESH today, so draw our own */

	AddMesh(hNexus, &VECTOR_ZERO );
	AddMesh(hChute, &VECTOR_ZERO );
	if (stage == 0)
		AddMesh(hNexusR, &RETRO_OFS);

	if (!AnimationsMade) MakeAnimations();

	CreateVariableDragElement (&brakes_proc, 1400.0, _V(0, 0.0, -68.0));
	CreateVariableDragElement (&chute_proc, 15400.0, _V(0, 0, -68.0));

	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */
	if (stage < 2)
		if (ph_first==NULL) 
			if (InitFuelLevel != 1.0)
				ph_first  = CreatePropellantResource(FIRST_PROP_MASS,FIRST_PROP_MASS*InitFuelLevel,1.0); 
			else
				ph_first  = CreatePropellantResource(FIRST_PROP_MASS,FIRST_PROP_MASS,1.0); 
	if (ph_att==NULL)  
		ph_att  = CreatePropellantResource(ATT_PROP_MASS,ATT_PROP_MASS,1.0); 
	if (ph_braking==NULL)  
		ph_braking  = CreatePropellantResource(RETRO_PROP_MASS,RETRO_PROP_MASS,1.0); 

//	SetDefaultPropellantResource (ph_first); 

	th_first[0] = CreateThruster ( _V(0,0,-11.7), _V(0,0,1), THRUST_FIRST_VAC, ph_first, ISP_FIRST_VAC, ISP_FIRST_SL);
//	AddExhaustStream (th_first[0], _V(0.72,0,-7), &contrail);
//	AddExhaustStream (th_first[0], _V(0.72,0,-7), &exhaust_main);

	thg_first = CreateThrusterGroup (th_first, 1, THGROUP_MAIN);

	for (int i = 0; i < 24; i++)	
		AddExhaust(th_first[0],pow(GetThrusterMax(th_first[0])/28,0.3333)*0.5,pow(GetThrusterMax(th_first[0])/28,0.3333)*0.035,_V((-12.5*cos(i*(6.28/24))),(-12.5*sin(i*(6.28/24))),-11.7),_V(0,0,-1),tex);
	
	AddExhaust(th_first[0],pow(GetThrusterMax(th_first[0])/28,0.3333)*0.5,pow(GetThrusterMax(th_first[0])/28,0.3333)*0.035,_V( 21.15,0.0,-13.6),_V(0,0,-1),tex);
	AddExhaust(th_first[0],pow(GetThrusterMax(th_first[0])/28,0.3333)*0.5,pow(GetThrusterMax(th_first[0])/28,0.3333)*0.035,_V(-21.15,0.0,-13.6),_V(0,0,-1),tex);
	AddExhaust(th_first[0],pow(GetThrusterMax(th_first[0])/28,0.3333)*0.5,pow(GetThrusterMax(th_first[0])/28,0.3333)*0.035,_V(0.0, 21.15,-13.6),_V(0,0,-1),tex);
	AddExhaust(th_first[0],pow(GetThrusterMax(th_first[0])/28,0.3333)*0.5,pow(GetThrusterMax(th_first[0])/28,0.3333)*0.035,_V(0.0,-21.15,-13.6),_V(0,0,-1),tex);

	if (stage > 0) {
		th_retro[0] = CreateThruster ( _V(0,0,18.1), _V(0,0,-1), THRUST_BRAKE_VAC, ph_braking, ISP_BRAKE_VAC, ISP_BRAKE_SL);
		thg_retro = CreateThrusterGroup (th_retro, 1, THGROUP_RETRO);

		for (int j = 0; j < 24; j++) {	
			AddExhaustStream (th_retro[0], _V((-8.0*cos(j*(6.28/24))),(-8.0*sin(j*(6.28/24))),18.1), &contrail);
			AddExhaustStream (th_retro[0], _V((-8.0*cos(j*(6.28/24))),(-8.0*sin(j*(6.28/24))),18.1), &exhaust_main);
		}
	//		AddExhaust(th_retro[0],pow(GetThrusterMax(th_retro[0])/24,0.3333)*0.5,pow(GetThrusterMax(th_retro[0])/24,0.3333)*0.035,_V((-8.0*cos(i*(6.28/24))),(-8.0*sin(i*(6.28/24))),18.1),_V(0,0,1),tex);

	}
	else {
		th_retro[0] = CreateThruster ( _V(0,0,18.1), _V(0,0,-1), THRUST_RETRO_VAC, ph_first, ISP_FIRST_VAC, ISP_FIRST_SL);
		thg_retro = CreateThrusterGroup (th_retro, 1, THGROUP_RETRO);
		AddExhaust(th_retro[0],pow(GetThrusterMax(th_retro[0])/28,0.3333)*0.5,pow(GetThrusterMax(th_retro[0])/28,0.3333)*0.035,_V( 14.5,0.0,26.1),_V(0,0,1),tex);
		AddExhaust(th_retro[0],pow(GetThrusterMax(th_retro[0])/28,0.3333)*0.5,pow(GetThrusterMax(th_retro[0])/28,0.3333)*0.035,_V(-14.5,0.0,26.1),_V(0,0,1),tex);
		AddExhaust(th_retro[0],pow(GetThrusterMax(th_retro[0])/28,0.3333)*0.5,pow(GetThrusterMax(th_retro[0])/28,0.3333)*0.035,_V(0.0, 14.5,26.1),_V(0,0,1),tex);
		AddExhaust(th_retro[0],pow(GetThrusterMax(th_retro[0])/28,0.3333)*0.5,pow(GetThrusterMax(th_retro[0])/28,0.3333)*0.035,_V(0.0,-14.5,26.1),_V(0,0,1),tex);

	}

	if (GetPropellantMass(ph_first) > 0.0) 
		SetAttControlsNexus(ph_first);
	else SetAttControlsNexus(ph_att);
	EnableTransponder(true);
}





/** Handle buffered key events sent to the vessel by orbiter. 
 *  @param dwKey The scan code of the key
 *  @param fPressed true if pressed, false if depressed
 *  @param keystate the current state of the keyboard
 *
 *  modified switch routine by dave rowbotham.
 */
int NEXUS::ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate)
{
	int result = 0;
	if(!fPressed && !KEYMOD_SHIFT(keystate) && !KEYMOD_CONTROL(keystate) && !KEYMOD_ALT(keystate))
	{
		switch(dwKey)
		{ 
		case OAPI_KEY_J:
			result = 1;

			/* CVE-lite function ! */
			/* IMPORTANT */
			/* call DeployPayload() to jettison the GPDS payloads in sequence */

			if (stage == 0)
				JettisonRetro();
			else {
				DeployPayload();

				/* IMPORTANT */
				/* Always re-set the stage after any kind of change */
				SetMainStage();
			}

			break;

		}
	}
	return result;
}



int NEXUS::ConsumeKey( const char *keystate )
{

	if (KEYMOD_SHIFT (keystate)) 
	{
						
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		return 0;
	}
	else 
	{ // unmodified keys
	
		return 0;

	}
	return 0;
}

void NEXUS::JettisonRetro ()
{
	stage++;
	VESSELSTATUS vs;
	GetStatus (vs);
	VECTOR3 ofs = RETRO_OFS;
	VECTOR3 vel = _V(0,0.0,2.0);
	VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
	Local2Rel (ofs, vs.rpos);
	GlobalRot (vel, rofs);
	vs.rvel.x = rvel.x+rofs.x;
	vs.rvel.y = rvel.y+rofs.y;
	vs.rvel.z = rvel.z+rofs.z;
	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	char name[256];
	strcpy (name, GetName()); strcat (name, "-Retro");
	VESSEL::Create (name, "Nexus_Retrosection", vs);
	SetMainStage();
//	PlayVesselWave3(MyID,PLAYJETT);
}


void NEXUS::SetAttControlsNexus(PROPELLANT_HANDLE ph_rcs)
{
	double dist = GetSize();

	THRUSTER_HANDLE th_rot[2];

	th_rot[0] = CreateThruster (_V(0,0, dist), _V(0, 1,0), MAX_ATT_FIRST, ph_rcs, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-dist), _V(0,-1,0), MAX_ATT_FIRST, ph_rcs, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHUP);

	th_rot[0] = CreateThruster (_V(0,0,-dist), _V(0,1,0),MAX_ATT_FIRST, ph_rcs, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0, dist), _V(0,1,0),MAX_ATT_FIRST, ph_rcs, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_UP);

	th_rot[0] = CreateThruster (_V(0,0,-dist), _V(0,-1,0),MAX_ATT_FIRST, ph_rcs, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0, dist), _V(0,-1,0),MAX_ATT_FIRST, ph_rcs, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_DOWN);

	th_rot[0] = CreateThruster (_V(0,0, dist), _V(0,-1,0), MAX_ATT_FIRST, ph_rcs, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-dist), _V(0, 1,0), MAX_ATT_FIRST, ph_rcs, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHDOWN);

	th_rot[0] = CreateThruster (_V(0,0, dist), _V(-1,0,0), MAX_ATT_FIRST, ph_rcs, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-dist), _V( 1,0,0), MAX_ATT_FIRST, ph_rcs, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWLEFT);

	th_rot[0] = CreateThruster (_V(0,0, dist), _V( 1,0,0), MAX_ATT_FIRST, ph_rcs, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-dist), _V(-1,0,0), MAX_ATT_FIRST, ph_rcs, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWRIGHT);

	th_rot[0] = CreateThruster (_V( dist,0.0,0.0), _V(-1,0,0), MAX_ATT_FIRST, ph_rcs, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_LEFT);
	th_rot[0] = CreateThruster (_V(-dist,0.0,0.0), _V( 1,0,0), MAX_ATT_FIRST, ph_rcs, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_RIGHT);

	th_rot[0] = CreateThruster (_V( dist*2,0,0), _V(0,-1,0), MAX_ATT_FIRST, ph_rcs, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(-dist*2,0,0), _V(0, 1,0), MAX_ATT_FIRST, ph_rcs, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKRIGHT);

	th_rot[0] = CreateThruster (_V( dist*2,0, 0), _V(0, 1,0), MAX_ATT_FIRST, ph_rcs, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(-dist*2,0,0), _V(0, -1,0), MAX_ATT_FIRST, ph_rcs, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKLEFT);

	th_rot[0] = CreateThruster (_V(0,0, -dist), _V(0,0,1), MAX_ATT_FIRST, ph_rcs, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_FORWARD);
	th_rot[0] = CreateThruster (_V(0,0, dist), _V(0,0,-1), MAX_ATT_FIRST, ph_rcs, ISP_FIRST_SL);
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_BACK);
}
