/**
*
* $Id: CVEL-Nexus.h,v 0.1 2004/11/20 18:03:45 erv Exp $
* (c) 2004 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/


#include "../../include/cvel.h"
#include "../../include/orbitersdk.h"
#include <cstdio>
#include <cmath>
#include <string>

#define InitFuelLevel 1

// All data from astronautix.com
const double ISP_FIRST_SL		=       3913.57; 
const double ISP_FIRST_VAC		=       4467.11; //***
const double ISP_BRAKE_SL		=     226.0*G; 
const double ISP_BRAKE_VAC		=     256.0*G; //***
const double THRUST_FIRST_VAC	=  353160000.0; //***
const double THRUST_BRAKE_VAC	=  103160000.0; //***
const double THRUST_RETRO_VAC	=    9160000.0; //***
const double MAX_ATT_FIRST		=    2565200.0/20.0;
const double FIRST_PROP_MASS	=   21950000.0; 
const double ATT_PROP_MASS		=      10000.0;
const double RETRO_PROP_MASS	=      60000.0;
const double NEXUS_DRYMASS		=    1980000.0; //***

const double BRAKES_OPERATING_SPEED = 0.0125;
const double CHUTE_OPERATING_SPEED = 0.25;
const double DEFLATE_SPEED = 0.025;

const VECTOR3 RETRO_OFS = _V(0,0,19.35);

/* make your own vessel class (named NEXUS here, for example)
   subclassed from COMMON_VESSEL_EX */

class NEXUS : public COMMON_VESSEL_EX {
public:
	NEXUS (OBJHANDLE hObj, int fmodel);
	~NEXUS();

	enum DoorStatus { DOOR_CLOSED, DOOR_OPEN, DOOR_CLOSING, DOOR_OPENING } brakes_status, chute_status, deflate_status;

	void TimeStep(  double simt ); // need a timestep loop to conduct the GPDS polling
	void VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount ); // must use VisualCreated() instead of ovcVisualCreated()
	void SetClassCaps(  FILEHANDLE cfg );
	void LoadState(  FILEHANDLE scn, void *vs );
	void SaveState(  FILEHANDLE scn );
	void UpdateAnimations();
	void ActivateBrakes (DoorStatus action);
	void RevertBrakes();
	void ActivateChute (DoorStatus action);
	void RevertChute();
	void ActivateDeflate (DoorStatus action);
	void RevertDeflate();
	void MakeAnimations();
	void JettisonRetro();

	/* CVE-lite requirement ! */
	// Create a function that can be called as many times as you like. It is
	// vital to have a function like this that cleans out all the
	// thrusters and meshes and draws/recreates them again.
	// This is so that you can respond immediately to GPDS requests.
	// Here I've called it SetMainStage() it can be Refresh() or anything you like.
	void SetMainStage(); 

	void SetAttControlsNexus(PROPELLANT_HANDLE ph_rcs);    // this is to set up the thrusters. I like to keep them separate.
	int  ConsumeKey(  const char *keystate );
	int  ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate);

// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkLoadStateEx (FILEHANDLE scn, void *vs);
	void clbkSaveState (FILEHANDLE scn);
	void clbkPostStep (double simt, double simdt, double mjd);
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);


protected:
	PROPELLANT_HANDLE ph_first, ph_att, ph_braking;
	THRUSTER_HANDLE   th_first[1], th_retro[1];
	THGROUP_HANDLE    thg_first,thg_retro,thg_pitchup, thg_pitchdown;
	MESHHANDLE		  hNexus,hNexusR,hChute;
	SURFHANDLE		  tex;
	UINT anim_brakes, anim_chute, anim_deflate;	
	double brakes_proc, chute_proc, deflate_proc;
	bool AnimationsMade;
	int stage;

};







/* basic vessel class setup */

void NEXUS::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetClassCaps(cfg);
}

void NEXUS::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	LoadState(  scn, vs );
}

void NEXUS::clbkSaveState (FILEHANDLE scn)
{
	SaveState(  scn );
}

void NEXUS::clbkPostStep (double simt, double simdt, double mjd)
{
	TimeStep(simt);
}

void NEXUS::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

void NEXUS::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

int NEXUS::clbkConsumeBufferedKey (DWORD dwKey, bool fPressed, char *keystate)
{
	int ret = ConsumeBufferedKey( dwKey, fPressed, keystate );
	return ret;
}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new NEXUS (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (NEXUS*)vessel;
}

